@@ -1,26 +1,26 @@
-registerCleanupFunction(async function cleanup() {
-  await Services.search.setDefault(originalEngine);
-  let engine = Services.search.getEngineByName("MozSearch");
-  await Services.search.removeEngine(engine);
-});
+const { SearchTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/SearchTestUtils.sys.mjs"
+);
+
+SearchTestUtils.init(this);
 
-let originalEngine;
 add_task(async function test_setup() {
   // Opening multiple windows on debug build takes too long time.
   requestLongerTimeout(10);
 
   // Stop search-engine loads from hitting the network
-  await Services.search.addEngineWithDetails("MozSearch", {
-    method: "GET",
-    template: "http://example.com/?q={searchTerms}",
-  });
-  let engine = Services.search.getEngineByName("MozSearch");
-  originalEngine = await Services.search.getDefault();
-  await Services.search.setDefault(engine);
+  await SearchTestUtils.installSearchExtension(
+    {
+      name: "MozSearch",
+      search_url: "https://example.com/",
+      search_url_get_params: "q={searchTerms}",
+    },
+    { setAsDefault: true }
+  );
 
   // Move New Window button to nav bar, to make it possible to drag and drop.
-  let { CustomizableUI } = ChromeUtils.import(
-    "resource:///modules/CustomizableUI.jsm"
+  let { CustomizableUI } = ChromeUtils.importESModule(
+    "moz-src:///browser/components/customizableui/CustomizableUI.sys.mjs"
   );
   let origPlacement = CustomizableUI.getPlacementOfWidget("new-window-button");
   if (!origPlacement || origPlacement.area != CustomizableUI.AREA_NAVBAR) {
@@ -30,15 +30,20 @@ add_task(async function test_setup() {
       0
     );
     CustomizableUI.ensureWidgetPlacedInWindow("new-window-button", window);
-    registerCleanupFunction(function() {
+    registerCleanupFunction(function () {
       CustomizableUI.removeWidgetFromArea("new-window-button");
     });
   }
+
+  CustomizableUI.addWidgetToArea("sidebar-button", "nav-bar");
+  registerCleanupFunction(() =>
+    CustomizableUI.removeWidgetFromArea("sidebar-button")
+  );
 });
 
 // New Window Button opens any link.
 add_task(async function single_url() {
-  await dropText("mochi.test/first", ["http://www.mochi.test/first"]);
+  await dropText("mochi.test/first", ["http://mochi.test/first"]);
 });
 add_task(async function single_javascript() {
   await dropText("javascript:'bad'", ["about:blank"]);
@@ -47,31 +52,31 @@ add_task(async function single_javascript_capital() {
   await dropText("jAvascript:'bad'", ["about:blank"]);
 });
 add_task(async function single_url2() {
-  await dropText("mochi.test/second", ["http://www.mochi.test/second"]);
+  await dropText("mochi.test/second", ["http://mochi.test/second"]);
 });
 add_task(async function single_data_url() {
   await dropText("data:text/html,bad", ["data:text/html,bad"]);
 });
 add_task(async function single_url3() {
-  await dropText("mochi.test/third", ["http://www.mochi.test/third"]);
+  await dropText("mochi.test/third", ["http://mochi.test/third"]);
 });
 
 // Single text/plain item, with multiple links.
 add_task(async function multiple_urls() {
   await dropText("mochi.test/1\nmochi.test/2", [
-    "http://www.mochi.test/1",
-    "http://www.mochi.test/2",
+    "http://mochi.test/1",
+    "http://mochi.test/2",
   ]);
 });
 add_task(async function multiple_urls_javascript() {
   await dropText("javascript:'bad1'\nmochi.test/3", [
     "about:blank",
-    "http://www.mochi.test/3",
+    "http://mochi.test/3",
   ]);
 });
 add_task(async function multiple_urls_data() {
   await dropText("mochi.test/4\ndata:text/html,bad1", [
-    "http://www.mochi.test/4",
+    "http://mochi.test/4",
     "data:text/html,bad1",
   ]);
 });
@@ -83,11 +88,7 @@ add_task(async function multiple_items_single_and_multiple_links() {
       [{ type: "text/plain", data: "mochi.test/5" }],
       [{ type: "text/plain", data: "mochi.test/6\nmochi.test/7" }],
     ],
-    [
-      "http://www.mochi.test/5",
-      "http://www.mochi.test/6",
-      "http://www.mochi.test/7",
-    ]
+    ["http://mochi.test/5", "http://mochi.test/6", "http://mochi.test/7"]
   );
 });
 
@@ -103,7 +104,7 @@ add_task(async function single_moz_url_multiple_links() {
         },
       ],
     ],
-    ["http://www.mochi.test/8", "http://www.mochi.test/9"]
+    ["http://mochi.test/8", "http://mochi.test/9"]
   );
 });
 
@@ -116,7 +117,7 @@ add_task(async function single_item_multiple_types() {
         { type: "text/x-moz-url", data: "mochi.test/11\nTITLE11" },
       ],
     ],
-    ["http://www.mochi.test/11"]
+    ["http://mochi.test/11"]
   );
 });
 
@@ -127,11 +128,11 @@ add_task(async function multiple_tabs_under_max() {
     urls.push("mochi.test/multi" + i);
   }
   await dropText(urls.join("\n"), [
-    "http://www.mochi.test/multi0",
-    "http://www.mochi.test/multi1",
-    "http://www.mochi.test/multi2",
-    "http://www.mochi.test/multi3",
-    "http://www.mochi.test/multi4",
+    "http://mochi.test/multi0",
+    "http://mochi.test/multi1",
+    "http://mochi.test/multi2",
+    "http://mochi.test/multi3",
+    "http://mochi.test/multi4",
   ]);
 });
 add_task(async function multiple_tabs_over_max_accept() {
@@ -146,11 +147,11 @@ add_task(async function multiple_tabs_over_max_accept() {
   await dropText(
     urls.join("\n"),
     [
-      "http://www.mochi.test/accept0",
-      "http://www.mochi.test/accept1",
-      "http://www.mochi.test/accept2",
-      "http://www.mochi.test/accept3",
-      "http://www.mochi.test/accept4",
+      "http://mochi.test/accept0",
+      "http://mochi.test/accept1",
+      "http://mochi.test/accept2",
+      "http://mochi.test/accept3",
+      "http://mochi.test/accept4",
     ],
     true
   );
@@ -183,16 +184,11 @@ function dropText(text, expectedURLs, ignoreFirstWindow = false) {
   );
 }
 
-async function drop(dragData, expectedURLs, ignoreFirstWindow = false) {
+async function drop(dragData, expectedURLs) {
   let dragDataString = JSON.stringify(dragData);
   info(
     `Starting test for dragData:${dragDataString}; expectedURLs.length:${expectedURLs.length}`
   );
-  let EventUtils = {};
-  Services.scriptloader.loadSubScript(
-    "chrome://mochikit/content/tests/SimpleTest/EventUtils.js",
-    EventUtils
-  );
 
   // Since synthesizeDrop triggers the srcElement, need to use another button
   // that should be visible.
@@ -201,9 +197,6 @@ async function drop(dragData, expectedURLs, ignoreFirstWindow = false) {
   let newWindowButton = document.getElementById("new-window-button");
   ok(newWindowButton, "New Window button exists");
 
-  let tmp = {};
-  ChromeUtils.import("resource://testing-common/TestUtils.jsm", tmp);
-
   let awaitDrop = BrowserTestUtils.waitForEvent(newWindowButton, "drop");
 
   let loadedPromises = expectedURLs.map(url =>
