@@ -15,8 +15,8 @@ function frameScript() {
   }
 
   let range = selection.getRangeAt(0);
-  const { FindContent } = ChromeUtils.import(
-    "resource://gre/modules/FindContent.jsm"
+  const { FindContent } = ChromeUtils.importESModule(
+    "resource://gre/modules/FindContent.sys.mjs"
   );
   let highlighter = new FindContent(docShell).highlighter;
   let r1 = content.parent.frameElement.getBoundingClientRect();
@@ -38,20 +38,18 @@ add_task(async function testFind() {
   async function background() {
     function awaitLoad(tabId, url) {
       return new Promise(resolve => {
-        browser.tabs.onUpdated.addListener(function listener(
-          tabId_,
-          changed,
-          tab
-        ) {
-          if (
-            tabId == tabId_ &&
-            changed.status == "complete" &&
-            tab.url == url
-          ) {
-            browser.tabs.onUpdated.removeListener(listener);
-            resolve();
+        browser.tabs.onUpdated.addListener(
+          function listener(tabId_, changed, tab) {
+            if (
+              tabId == tabId_ &&
+              changed.status == "complete" &&
+              tab.url == url
+            ) {
+              browser.tabs.onUpdated.removeListener(listener);
+              resolve();
+            }
           }
-        });
+        );
       });
     }
 
@@ -80,7 +78,7 @@ add_task(async function testFind() {
     browser.test.log(
       "Test that the text found in the top window and nested frames corresponds to the proper position."
     );
-    let terms = ["Banana", "bAnana", "baNana", "banAna", "banaNa", "bananA"];
+    let terms = ["Bánana", "bAnana", "baNana", "banAna", "banaNa", "bananA"];
     for (let i = 0; i < terms.length; i++) {
       browser.test.assertEq(
         terms[i],
@@ -97,6 +95,14 @@ add_task(async function testFind() {
     browser.test.assertEq(1, data.count, "The number of matches found:");
     browser.test.assertEq("baNana", data.rangeData[0].text, "The text found:");
 
+    browser.test.log("Test that diacritic sensitive match works properly.");
+    data = await browser.find.find("bánana", {
+      matchDiacritics: true,
+      includeRangeData: true,
+    });
+    browser.test.assertEq(1, data.count, "The number of matches found:");
+    browser.test.assertEq("Bánana", data.rangeData[0].text, "The text found:");
+
     browser.test.log("Test that case insensitive match works properly.");
     data = await browser.find.find("banana", { caseSensitive: false });
     browser.test.assertEq(6, data.count, "The number of matches found:");
@@ -113,33 +119,33 @@ add_task(async function testFind() {
       {
         framePos: 0,
         text: "example",
-        startTextNodePos: 15,
+        startTextNodePos: 16,
         startOffset: 11,
-        endTextNodePos: 15,
+        endTextNodePos: 16,
         endOffset: 18,
       },
       {
         framePos: 0,
         text: "example",
-        startTextNodePos: 15,
+        startTextNodePos: 16,
         startOffset: 25,
-        endTextNodePos: 15,
+        endTextNodePos: 16,
         endOffset: 32,
       },
       {
         framePos: 0,
         text: "example",
-        startTextNodePos: 18,
+        startTextNodePos: 19,
         startOffset: 6,
-        endTextNodePos: 18,
+        endTextNodePos: 19,
         endOffset: 13,
       },
       {
         framePos: 0,
         text: "example",
-        startTextNodePos: 20,
+        startTextNodePos: 21,
         startOffset: 3,
-        endTextNodePos: 20,
+        endTextNodePos: 21,
         endOffset: 10,
       },
       {
@@ -289,20 +295,18 @@ add_task(async function testRemoveHighlighting() {
   async function background() {
     function awaitLoad(tabId, url) {
       return new Promise(resolve => {
-        browser.tabs.onUpdated.addListener(function listener(
-          tabId_,
-          changed,
-          tab
-        ) {
-          if (
-            tabId == tabId_ &&
-            changed.status == "complete" &&
-            tab.url == url
-          ) {
-            browser.tabs.onUpdated.removeListener(listener);
-            resolve();
+        browser.tabs.onUpdated.addListener(
+          function listener(tabId_, changed, tab) {
+            if (
+              tabId == tabId_ &&
+              changed.status == "complete" &&
+              tab.url == url
+            ) {
+              browser.tabs.onUpdated.removeListener(listener);
+              resolve();
+            }
           }
-        });
+        );
       });
     }
 
@@ -404,7 +408,7 @@ add_task(async function testIncognitoFind() {
   let privateWin = await BrowserTestUtils.openNewBrowserWindow({
     private: true,
   });
-  BrowserTestUtils.loadURI(
+  BrowserTestUtils.startLoadingURIString(
     privateWin.gBrowser.selectedBrowser,
     "http://example.com"
   );
@@ -439,7 +443,10 @@ add_task(async function testIncognitoFindAllowed() {
   let privateWin = await BrowserTestUtils.openNewBrowserWindow({
     private: true,
   });
-  BrowserTestUtils.loadURI(privateWin.gBrowser.selectedBrowser, url);
+  BrowserTestUtils.startLoadingURIString(
+    privateWin.gBrowser.selectedBrowser,
+    url
+  );
   await BrowserTestUtils.browserLoaded(privateWin.gBrowser.selectedBrowser);
 
   let extension = ExtensionTestUtils.loadExtension({
