@@ -15,18 +15,17 @@ const MAX_RESULT_COUNT = UrlbarPrefs.get("maxRichResults");
 let suggestionsEngine;
 let expectedFormHistoryResults = [];
 
-add_setup(async function() {
-  suggestionsEngine = await SearchTestUtils.promiseNewSearchEngine(
-    getRootDirectory(gTestPath) + SUGGESTIONS_ENGINE_NAME
-  );
-
-  let oldDefaultEngine = await Services.search.getDefault();
-  await SearchTestUtils.installSearchExtension({
-    name: DEFAULT_ENGINE_NAME,
-    keyword: "@test",
+add_setup(async function () {
+  suggestionsEngine = await SearchTestUtils.installOpenSearchEngine({
+    url: getRootDirectory(gTestPath) + SUGGESTIONS_ENGINE_NAME,
   });
-  await Services.search.setDefault(
-    Services.search.getEngineByName(DEFAULT_ENGINE_NAME)
+
+  await SearchTestUtils.installSearchExtension(
+    {
+      name: DEFAULT_ENGINE_NAME,
+      keyword: "@test",
+    },
+    { setAsDefault: true }
   );
   await Services.search.moveEngine(suggestionsEngine, 0);
 
@@ -61,150 +60,24 @@ add_setup(async function() {
   ]);
 
   registerCleanupFunction(async () => {
-    await Services.search.setDefault(oldDefaultEngine);
     await UrlbarTestUtils.formHistory.clear();
   });
 
   await SpecialPowers.pushPrefEnv({
-    set: [["browser.search.separatePrivateDefault.ui.enabled", false]],
+    set: [
+      ["browser.search.separatePrivateDefault.ui.enabled", false],
+      ["browser.urlbar.scotchBonnet.enableOverride", false],
+      ["browser.urlbar.suggest.trending", false],
+      ["browser.urlbar.suggest.recentsearches", false],
+    ],
   });
 });
 
-add_task(async function emptySearch() {
-  await BrowserTestUtils.withNewTab("about:robots", async function(browser) {
-    await SpecialPowers.pushPrefEnv({
-      set: [["browser.urlbar.update2.emptySearchBehavior", 2]],
-    });
-    await UrlbarTestUtils.promiseAutocompleteResultPopup({
-      window,
-      value: "",
-    });
-    await UrlbarTestUtils.enterSearchMode(window);
-    Assert.equal(gURLBar.value, "", "Urlbar value should be cleared.");
-    // For the empty search case, we expect to get the form history relative to
-    // the picked engine and no heuristic.
-    await checkResults(expectedFormHistoryResults);
-    await UrlbarTestUtils.exitSearchMode(window, { clickClose: true });
-    await UrlbarTestUtils.promisePopupClose(window);
-    await SpecialPowers.popPrefEnv();
-  });
-});
-
-add_task(async function emptySearch_withRestyledHistory() {
-  // URLs with the same host as the search engine.
-  await PlacesTestUtils.addVisits([
-    `http://mochi.test/`,
-    `http://mochi.test/redirect`,
-    // Should not be returned because it's a redirect target.
-    {
-      uri: `http://mochi.test/target`,
-      transition: PlacesUtils.history.TRANSITIONS.REDIRECT_TEMPORARY,
-      referrer: `http://mochi.test/redirect`,
-    },
-    // Can be restyled and dupes form history.
-    "http://mochi.test:8888/?terms=hello+formHistory+0",
-    // Can be restyled but does not dupe form history.
-    "http://mochi.test:8888/?terms=ciao",
-  ]);
-  await BrowserTestUtils.withNewTab("about:robots", async function(browser) {
-    await SpecialPowers.pushPrefEnv({
-      set: [["browser.urlbar.update2.emptySearchBehavior", 2]],
-    });
-    await UrlbarTestUtils.promiseAutocompleteResultPopup({
-      window,
-      value: "",
-    });
-    await UrlbarTestUtils.enterSearchMode(window);
-    Assert.equal(gURLBar.value, "", "Urlbar value should be cleared.");
-    // For the empty search case, we expect to get the form history relative to
-    // the picked engine, history without redirects, and no heuristic.
-    await checkResults([
-      {
-        heuristic: false,
-        type: UrlbarUtils.RESULT_TYPE.SEARCH,
-        source: UrlbarUtils.RESULT_SOURCE.HISTORY,
-        searchParams: {
-          suggestion: "ciao",
-          engine: suggestionsEngine.name,
-        },
-      },
-      ...expectedFormHistoryResults.slice(0, MAX_RESULT_COUNT - 3),
-      {
-        heuristic: false,
-        type: UrlbarUtils.RESULT_TYPE.URL,
-        source: UrlbarUtils.RESULT_SOURCE.HISTORY,
-        url: `http://mochi.test/redirect`,
-      },
-      {
-        heuristic: false,
-        type: UrlbarUtils.RESULT_TYPE.URL,
-        source: UrlbarUtils.RESULT_SOURCE.HISTORY,
-        url: `http://mochi.test/`,
-      },
-    ]);
-
-    await UrlbarTestUtils.exitSearchMode(window, { clickClose: true });
-    await UrlbarTestUtils.promisePopupClose(window);
-    await SpecialPowers.popPrefEnv();
-  });
-
-  await PlacesUtils.history.clear();
-});
-
-add_task(async function emptySearch_withRestyledHistory_noSearchHistory() {
-  // URLs with the same host as the search engine.
-  await PlacesTestUtils.addVisits([
-    `http://mochi.test/`,
-    `http://mochi.test/redirect`,
-    // Can be restyled but does not dupe form history.
-    "http://mochi.test:8888/?terms=ciao",
-  ]);
-  await BrowserTestUtils.withNewTab("about:robots", async function(browser) {
-    await SpecialPowers.pushPrefEnv({
-      set: [
-        ["browser.urlbar.update2.emptySearchBehavior", 2],
-        ["browser.urlbar.maxHistoricalSearchSuggestions", 0],
-      ],
-    });
-    await UrlbarTestUtils.promiseAutocompleteResultPopup({
-      window,
-      value: "",
-    });
-    await UrlbarTestUtils.enterSearchMode(window);
-    Assert.equal(gURLBar.value, "", "Urlbar value should be cleared.");
-    // maxHistoricalSearchSuggestions == 0, so form history should not be
-    // present.
-    await checkResults([
-      {
-        heuristic: false,
-        type: UrlbarUtils.RESULT_TYPE.URL,
-        source: UrlbarUtils.RESULT_SOURCE.HISTORY,
-        url: `http://mochi.test/redirect`,
-      },
-      {
-        heuristic: false,
-        type: UrlbarUtils.RESULT_TYPE.URL,
-        source: UrlbarUtils.RESULT_SOURCE.HISTORY,
-        url: `http://mochi.test/`,
-      },
-    ]);
-
-    await UrlbarTestUtils.exitSearchMode(window, { clickClose: true });
-    await UrlbarTestUtils.promisePopupClose(window);
-    await SpecialPowers.popPrefEnv();
-  });
-
-  await PlacesUtils.history.clear();
-});
-
 add_task(async function emptySearch_behavior() {
   // URLs with the same host as the search engine.
   await PlacesTestUtils.addVisits([`http://mochi.test/`]);
 
-  await BrowserTestUtils.withNewTab("about:robots", async function(browser) {
-    await SpecialPowers.pushPrefEnv({
-      set: [["browser.urlbar.update2.emptySearchBehavior", 0]],
-    });
+  await BrowserTestUtils.withNewTab("about:robots", async function () {
     await UrlbarTestUtils.promiseAutocompleteResultPopup({
       window,
       value: "",
@@ -239,25 +112,6 @@ add_task(async function emptySearch_behavior() {
       },
     ]);
     await UrlbarTestUtils.promisePopupClose(window);
-    await SpecialPowers.popPrefEnv();
-  });
-
-  await BrowserTestUtils.withNewTab("about:robots", async function(browser) {
-    await SpecialPowers.pushPrefEnv({
-      set: [["browser.urlbar.update2.emptySearchBehavior", 1]],
-    });
-    await UrlbarTestUtils.promiseAutocompleteResultPopup({
-      window,
-      value: "",
-    });
-    await UrlbarTestUtils.enterSearchMode(window);
-    Assert.equal(gURLBar.value, "", "Urlbar value should be cleared.");
-    // For the empty search case, we expect to get the form history relative to
-    // the picked engine, history without redirects, and no heuristic.
-    await checkResults([...expectedFormHistoryResults]);
-    await UrlbarTestUtils.exitSearchMode(window, { clickClose: true });
-    await UrlbarTestUtils.promisePopupClose(window);
-    await SpecialPowers.popPrefEnv();
   });
 
   await PlacesUtils.history.clear();
@@ -266,10 +120,7 @@ add_task(async function emptySearch_behavior() {
 add_task(async function emptySearch_local() {
   await PlacesTestUtils.addVisits([`http://mochi.test/`]);
 
-  await BrowserTestUtils.withNewTab("about:robots", async function(browser) {
-    await SpecialPowers.pushPrefEnv({
-      set: [["browser.urlbar.update2.emptySearchBehavior", 0]],
-    });
+  await BrowserTestUtils.withNewTab("about:robots", async function () {
     await UrlbarTestUtils.promiseAutocompleteResultPopup({
       window,
       value: "",
@@ -278,8 +129,7 @@ add_task(async function emptySearch_local() {
       source: UrlbarUtils.RESULT_SOURCE.HISTORY,
     });
     Assert.equal(gURLBar.value, "", "Urlbar value should be cleared.");
-    // Even when emptySearchBehavior is 0, we still show the user's most frecent
-    // history for an empty search.
+    // Show the user's most frecent history for an empty search.
     await checkResults([
       {
         heuristic: false,
@@ -290,14 +140,13 @@ add_task(async function emptySearch_local() {
     ]);
     await UrlbarTestUtils.exitSearchMode(window, { clickClose: true });
     await UrlbarTestUtils.promisePopupClose(window);
-    await SpecialPowers.popPrefEnv();
   });
 
   await PlacesUtils.history.clear();
 });
 
 add_task(async function nonEmptySearch() {
-  await BrowserTestUtils.withNewTab("about:robots", async function(browser) {
+  await BrowserTestUtils.withNewTab("about:robots", async function () {
     let query = "hello";
     await UrlbarTestUtils.promiseAutocompleteResultPopup({
       window,
@@ -345,7 +194,7 @@ add_task(async function nonEmptySearch() {
 });
 
 add_task(async function nonEmptySearch_nonMatching() {
-  await BrowserTestUtils.withNewTab("about:robots", async function(browser) {
+  await BrowserTestUtils.withNewTab("about:robots", async function () {
     let query = "ciao";
     await UrlbarTestUtils.promiseAutocompleteResultPopup({
       window,
@@ -393,9 +242,9 @@ add_task(async function nonEmptySearch_nonMatching() {
 });
 
 add_task(async function nonEmptySearch_withHistory() {
-  let manySuggestionsEngine = await SearchTestUtils.promiseNewSearchEngine(
-    getRootDirectory(gTestPath) + MANY_SUGGESTIONS_ENGINE_NAME
-  );
+  let manySuggestionsEngine = await SearchTestUtils.installOpenSearchEngine({
+    url: getRootDirectory(gTestPath) + MANY_SUGGESTIONS_ENGINE_NAME,
+  });
   // URLs with the same host as the search engine.
   let query = "ciao";
   await PlacesTestUtils.addVisits([
@@ -419,7 +268,7 @@ add_task(async function nonEmptySearch_withHistory() {
     };
   }
 
-  await BrowserTestUtils.withNewTab("about:robots", async function(browser) {
+  await BrowserTestUtils.withNewTab("about:robots", async function () {
     await UrlbarTestUtils.promiseAutocompleteResultPopup({
       window,
       value: query,
@@ -517,7 +366,7 @@ add_task(async function nonEmptySearch_withHistory() {
 });
 
 add_task(async function nonEmptySearch_url() {
-  await BrowserTestUtils.withNewTab("about:robots", async function(browser) {
+  await BrowserTestUtils.withNewTab("about:robots", async function () {
     let query = "http://www.example.com/";
     await UrlbarTestUtils.promiseAutocompleteResultPopup({
       window,
