@@ -2,15 +2,24 @@
 /* import-globals-from ../../../../../testing/mochitest/tests/SimpleTest/EventUtils.js */
 /* import-globals-from ../../../../../toolkit/components/satchel/test/satchel_common.js */
 /* eslint-disable no-unused-vars */
+// Despite a use of `spawnChrome` and thus ChromeUtils, we can't use isInstance
+// here as it gets used in plain mochitests which don't have the ChromeOnly
+// APIs for it.
+/* eslint-disable mozilla/use-isInstance */
 
 "use strict";
 
 let formFillChromeScript;
 let defaultTextColor;
+let defaultDisabledTextColor;
 let expectingPopup = null;
 
-const { FormAutofillUtils } = SpecialPowers.Cu.import(
-  "resource://formautofill/FormAutofillUtils.jsm"
+const { FormAutofillUtils } = SpecialPowers.ChromeUtils.importESModule(
+  "resource://gre/modules/shared/FormAutofillUtils.sys.mjs"
+);
+
+const { OSKeyStore } = SpecialPowers.ChromeUtils.importESModule(
+  "resource://gre/modules/OSKeyStore.sys.mjs"
 );
 
 async function sleep(ms = 500, reason = "Intentionally wait for UI ready") {
@@ -79,8 +88,9 @@ function clickOnElement(selector) {
   SimpleTest.executeSoon(() => element.click());
 }
 
-// The equivalent helper function to getAdaptedProfiles in FormAutofillHandler.jsm that
-// transforms the given profile to expected filled profile.
+// The equivalent helper function to getAdaptedProfiles in
+// FormAutofillSection.sys.mjs that transforms the given profile to expected
+// filled profile.
 function _getAdaptedProfile(profile) {
   const adaptedProfile = Object.assign({}, profile);
 
@@ -93,34 +103,35 @@ function _getAdaptedProfile(profile) {
   return adaptedProfile;
 }
 
-// We could not get ManuallyManagedState of element now, so directly check if
-// filter and text color style are applied.
 async function checkFieldHighlighted(elem, expectedValue) {
   let isHighlightApplied;
   await SimpleTest.promiseWaitForCondition(function checkHighlight() {
-    const computedStyle = window.getComputedStyle(elem);
-    isHighlightApplied = computedStyle.getPropertyValue("filter") !== "none";
+    isHighlightApplied = elem.matches(":autofill");
     return isHighlightApplied === expectedValue;
   }, `Checking #${elem.id} highlight style`);
 
   is(isHighlightApplied, expectedValue, `Checking #${elem.id} highlight style`);
 }
 
-async function checkFieldPreview(elem, expectedValue) {
-  is(
-    SpecialPowers.wrap(elem).previewValue,
-    expectedValue,
-    `Checking #${elem.id} previewValue`
-  );
-  let isTextColorApplied;
-  await SimpleTest.promiseWaitForCondition(function checkPreview() {
-    const computedStyle = window.getComputedStyle(elem);
-    isTextColorApplied =
-      computedStyle.getPropertyValue("color") !== defaultTextColor;
-    return isTextColorApplied === !!expectedValue;
-  }, `Checking #${elem.id} preview style`);
+async function checkFormFieldsStyle(profile, isPreviewing = true) {
+  const elems = document.querySelectorAll("input, select");
 
-  is(isTextColorApplied, !!expectedValue, `Checking #${elem.id} preview style`);
+  for (const elem of elems) {
+    let fillableValue;
+    let previewValue;
+    let isElementEligible =
+      FormAutofillUtils.isCreditCardOrAddressFieldType(elem) &&
+      FormAutofillUtils.isFieldAutofillable(elem);
+    if (!isElementEligible) {
+      fillableValue = "";
+      previewValue = "";
+    } else {
+      fillableValue = profile && profile[elem.id];
+      previewValue =
+        (isPreviewing && fillableValue?.toString().replaceAll("*", "•")) || "";
+    }
+    await checkFieldHighlighted(elem, !!fillableValue);
+  }
 }
 
 function checkFieldValue(elem, expectedValue) {
@@ -131,13 +142,8 @@ function checkFieldValue(elem, expectedValue) {
 }
 
 async function triggerAutofillAndCheckProfile(profile) {
-  const adaptedProfile = _getAdaptedProfile(profile);
+  let adaptedProfile = _getAdaptedProfile(profile);
   const promises = [];
-
-  await SpecialPowers.pushPrefEnv({
-    set: [["dom.input_events.beforeinput.enabled", true]],
-  });
-
   for (const [fieldName, value] of Object.entries(adaptedProfile)) {
     info(`triggerAutofillAndCheckProfile: ${fieldName}`);
     const element = document.getElementById(fieldName);
@@ -173,8 +179,10 @@ async function triggerAutofillAndCheckProfile(profile) {
             );
             is(
               event.cancelable,
-              true,
-              `"beforeinput" event should be cancelable on ${element.tagName}`
+              SpecialPowers.getBoolPref(
+                "dom.input_event.allow_to_cancel_set_user_input"
+              ),
+              `"beforeinput" event should be cancelable on ${element.tagName} unless it's suppressed by the pref`
             );
             is(
               event.bubbles,
@@ -188,7 +196,10 @@ async function triggerAutofillAndCheckProfile(profile) {
         element.addEventListener(
           "input",
           event => {
-            if (element.tagName == "INPUT" && element.type == "text") {
+            if (
+              (element.tagName == "INPUT" && element.type == "text") ||
+              element.tagName == "TEXTAREA"
+            ) {
               if (hadEditor) {
                 ok(
                   beforeInputFired,
@@ -271,12 +282,19 @@ async function onStorageChanged(type) {
   });
 }
 
-function checkMenuEntries(expectedValues, isFormAutofillResult = true) {
-  let actualValues = getMenuEntries();
-  // Expect one more item would appear at the bottom as the footer if the result is from form autofill.
-  let expectedLength = isFormAutofillResult
-    ? expectedValues.length + 1
-    : expectedValues.length;
+function makeAddressComment({ primary, secondary, status }) {
+  return JSON.stringify({
+    primary,
+    secondary,
+    status,
+    ariaLabel: primary + " " + secondary + " " + status,
+  });
+}
+
+// Compare the labels on the autocomplete menu items to the expected labels.
+function checkMenuEntries(expectedValues, extraRows = 1) {
+  let actualValues = getMenuEntries().labels;
+  let expectedLength = expectedValues.length + extraRows;
 
   is(actualValues.length, expectedLength, " Checking length of expected menu");
   for (let i = 0; i < expectedValues.length; i++) {
@@ -284,6 +302,26 @@ function checkMenuEntries(expectedValues, isFormAutofillResult = true) {
   }
 }
 
+// Compare the comment on the autocomplete menu items to the expected comment.
+// The profile field is not compared.
+function checkMenuEntriesComment(expectedValues, extraRows = 1) {
+  let actualValues = getMenuEntries().comments;
+  let expectedLength = expectedValues.length + extraRows;
+
+  is(actualValues.length, expectedLength, " Checking length of expected menu");
+  for (let i = 0; i < expectedValues.length; i++) {
+    const expectedValue = JSON.parse(expectedValues[i]);
+    const actualValue = JSON.parse(actualValues[i]);
+    for (const [key, value] of Object.entries(expectedValue)) {
+      is(
+        actualValue[key],
+        value,
+        ` Checking menu entry #${i}, ${key} should be the same`
+      );
+    }
+  }
+}
+
 function invokeAsyncChromeTask(message, payload = {}) {
   info(`expecting the chrome task finished: ${message}`);
   return formFillChromeScript.sendQuery(message, payload);
@@ -346,18 +384,79 @@ async function canTestOSKeyStoreLogin() {
   return canTest;
 }
 
+/**
+ * This function should be used along with the `waitForOSKeyStoreLogin` API.
+ * See the comment in `waitForOSKeyStoreLogin` for more details.
+ */
+async function waitForOSKeyStoreLoginTestSetupComplete() {
+  if (
+    !(await SpecialPowers.spawnChrome([], () => {
+      // Need to re-import this because we're running in the parent.
+      // eslint-disable-next-line no-shadow
+      const { FormAutofillUtils } = ChromeUtils.importESModule(
+        "resource://gre/modules/shared/FormAutofillUtils.sys.mjs"
+      );
+
+      return FormAutofillUtils.getOSAuthEnabled();
+    }))
+  ) {
+    return;
+  }
+
+  await SimpleTest.promiseWaitForCondition(async () => {
+    return await SpecialPowers.spawnChrome([], () => {
+      const { OSKeyStoreTestUtils } = ChromeUtils.importESModule(
+        "resource://testing-common/OSKeyStoreTestUtils.sys.mjs"
+      );
+
+      return Services.prefs.getStringPref(
+        OSKeyStoreTestUtils.TEST_ONLY_REAUTH,
+        ""
+      );
+    });
+  });
+}
+
+/**
+ * This API returns a promise that will be resolved when
+ * `waitForOSKeyStoreLogin` in `OSKeyStoreTestUtils.sys.mjs` completes.
+ * It is common to use it as follows:
+ *   const promise = waitForOSKeyStoreLogin();
+ *   triggerOSReauth();  // Code that triggers OS re-authentication
+ *   await promise;
+ *
+ * However, the timing to switch to using test OS re-auth after calling this
+ * function is asynchronous, which means triggering OS re-auth right after
+ * this API may still activate the real OS re-auth popup. To avoid that, you
+ * need to call `await waitForOSKeyStoreLoginTestSetupComplete()` before
+ * triggering OS re-auth.
+ */
 async function waitForOSKeyStoreLogin(login = false) {
+  // Need to fetch this from the parent in order for it to be correct.
+  if (
+    !(await SpecialPowers.spawnChrome([], () => {
+      // Need to re-import this because we're running in the parent.
+      // eslint-disable-next-line no-shadow
+      const { FormAutofillUtils } = ChromeUtils.importESModule(
+        "resource://gre/modules/shared/FormAutofillUtils.sys.mjs"
+      );
+
+      return FormAutofillUtils.getOSAuthEnabled();
+    }))
+  ) {
+    return;
+  }
+
   await invokeAsyncChromeTask("FormAutofillTest:OSKeyStoreLogin", { login });
 }
 
 function patchRecordCCNumber(record) {
-  const number = record["cc-number"];
-  const ccNumberFmt = {
-    affix: "****",
-    label: number.substr(-4),
-  };
+  const ccNumberFmt = "****" + record.cc["cc-number"].substr(-4);
 
-  return Object.assign({}, record, { ccNumberFmt });
+  return {
+    cc: Object.assign({}, record.cc, { ccNumberFmt }),
+    expected: record.expected,
+  };
 }
 
 // Utils for registerPopupShownListener(in satchel_common.js) that handles dropdown popup
@@ -393,9 +492,10 @@ function initPopupListener() {
 }
 
 async function triggerPopupAndHoverItem(fieldSelector, selectIndex) {
+  const promise = expectPopup();
   await focusAndWaitForFieldsIdentified(fieldSelector);
   synthesizeKey("KEY_ArrowDown");
-  await expectPopup();
+  await promise;
   for (let i = 0; i <= selectIndex; i++) {
     synthesizeKey("KEY_ArrowDown");
   }
@@ -415,7 +515,7 @@ function formAutoFillCommonSetup() {
     }
   });
 
-  add_task(async function setup() {
+  add_setup(async () => {
     info(`expecting the storage setup`);
     await formFillChromeScript.sendQuery("setup");
   });
@@ -430,13 +530,29 @@ function formAutoFillCommonSetup() {
 
   document.addEventListener(
     "DOMContentLoaded",
-    function() {
+    function () {
       defaultTextColor = window
         .getComputedStyle(document.querySelector("input"))
         .getPropertyValue("color");
+
+      // This is needed for test_formautofill_preview_highlight.html to work properly
+      let disabledInput = document.querySelector(`input[disabled]`);
+      if (disabledInput) {
+        defaultDisabledTextColor = window
+          .getComputedStyle(disabledInput)
+          .getPropertyValue("color");
+      }
     },
     { once: true }
   );
 }
 
+/*
+ * Extremely over-simplified detection of card type from card number just for
+ * our tests. This is needed to test the aria-label of credit card menu entries.
+ */
+function getCCTypeName(creditCard) {
+  return creditCard["cc-number"][0] == "4" ? "Visa" : "MasterCard";
+}
+
 formAutoFillCommonSetup();
