@@ -1,20 +1,14 @@
 "use strict";
 
-const { FieldScanner } = ChromeUtils.import(
-  "resource://autofill/FormAutofillHeuristics.jsm"
+var { FormAutofillHeuristics } = ChromeUtils.importESModule(
+  "resource://gre/modules/shared/FormAutofillHeuristics.sys.mjs"
+);
+var { LabelUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/shared/LabelUtils.sys.mjs"
+);
+var { FormAutofill } = ChromeUtils.importESModule(
+  "resource://autofill/FormAutofill.sys.mjs"
 );
-var FormAutofillHeuristics, LabelUtils, FormAutofill;
-add_task(async function() {
-  ({ FormAutofillHeuristics } = ChromeUtils.import(
-    "resource://autofill/FormAutofillHeuristics.jsm"
-  ));
-  ({ LabelUtils } = ChromeUtils.import(
-    "resource://autofill/FormAutofillUtils.jsm"
-  ));
-  ({ FormAutofill } = ChromeUtils.import(
-    "resource://autofill/FormAutofill.jsm"
-  ));
-});
 
 const TESTCASES = [
   {
@@ -25,12 +19,7 @@ const TESTCASES = [
                  </label>
                </form>`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "email",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["email", {}],
   },
   {
     description:
@@ -40,12 +29,7 @@ const TESTCASES = [
                  <input id="targetElement" type="text">
                </form>`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "email",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["email", {}],
   },
   {
     description: "A label element contains span element",
@@ -54,34 +38,19 @@ const TESTCASES = [
                  <input id="targetElement" type="text">
                </form>`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "email",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["email", {}],
   },
   {
     description: "The signature in 'name' attr of an input",
     document: `<input id="targetElement" name="email" type="text">`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "email",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["email", {}],
   },
   {
     description: "The signature in 'id' attr of an input",
     document: `<input id="targetElement_email" name="tel" type="text">`,
     elementId: "targetElement_email",
-    expectedReturnValue: {
-      fieldName: "email",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["email", {}],
   },
   {
     description: "Select element in a label element",
@@ -91,36 +60,21 @@ const TESTCASES = [
                  </label>
                </form>`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "address-level1",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["address-level1", {}],
   },
   {
     description: "A select element without a form wrapped",
     document: `<label for="targetElement">State</label>
                <select id="targetElement"></select>`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "address-level1",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["address-level1", {}],
   },
   {
     description: "address line input",
     document: `<label for="targetElement">street</label>
                <input id="targetElement" type="text">`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "street-address",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["street-address", {}],
   },
   {
     description: "CJK character - Traditional Chinese",
@@ -128,12 +82,7 @@ const TESTCASES = [
                  <input id="targetElement" />
                </label>`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "postal-code",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["postal-code", {}],
   },
   {
     description: "CJK character - Japanese",
@@ -141,12 +90,7 @@ const TESTCASES = [
                  <input id="targetElement" />
                </label>`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "postal-code",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["postal-code", {}],
   },
   {
     description: "CJK character - Korean",
@@ -154,51 +98,31 @@ const TESTCASES = [
                  <input id="targetElement" />
                </label>`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "postal-code",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["postal-code", {}],
   },
   {
     description: "",
     document: `<input id="targetElement" name="fullname">`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "name",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["name", {}],
   },
   {
     description: 'input element with "submit" type',
     document: `<input id="targetElement" type="submit" />`,
     elementId: "targetElement",
-    expectedReturnValue: null,
+    expectedReturnValue: [[], {}],
   },
   {
     description: "The signature in 'name' attr of an email input",
     document: `<input id="targetElement" name="email" type="number">`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "email",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["email", {}],
   },
   {
     description: 'input element with "email" type',
     document: `<input id="targetElement" type="email" />`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "email",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["email", {}],
   },
   {
     description: "Exclude United State string",
@@ -206,7 +130,7 @@ const TESTCASES = [
                  <input id="targetElement" />
                </label>`,
     elementId: "targetElement",
-    expectedReturnValue: null,
+    expectedReturnValue: [[], {}],
   },
   {
     description: '"County" field with "United State" string',
@@ -214,12 +138,7 @@ const TESTCASES = [
                  <input id="targetElement" />
                </label>`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "address-level1",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["address-level1", {}],
   },
   {
     description: '"city" field with double "United State" string',
@@ -227,12 +146,7 @@ const TESTCASES = [
                  <input id="targetElement" />
                </label>`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "address-level2",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["address-level2", {}],
   },
   {
     description: "Verify credit card number",
@@ -241,12 +155,7 @@ const TESTCASES = [
                  <input id="targetElement" type="text">
                </form>`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "cc-number",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["cc-number", { fathomConfidence: 1 }],
   },
   {
     description: "Identify credit card type field",
@@ -255,12 +164,7 @@ const TESTCASES = [
                  <input id="targetElement" type="text">
                </form>`,
     elementId: "targetElement",
-    expectedReturnValue: {
-      fieldName: "cc-type",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["cc-type", {}],
   },
   {
     description: `Identify address field when contained in a form with autocomplete="off"`,
@@ -268,17 +172,49 @@ const TESTCASES = [
                 <input id="given-name">
                </form>`,
     elementId: "given-name",
-    expectedReturnValue: {
-      fieldName: "given-name",
-      section: "",
-      addressType: "",
-      contactType: "",
-    },
+    expectedReturnValue: ["given-name", {}],
+  },
+  {
+    description: `Identify address field that has a placeholder but no label associated with it`,
+    document: `<form>
+                <input id="targetElement" placeholder="Name">
+               </form>`,
+    elementId: "targetElement",
+    expectedReturnValue: ["name", {}],
+  },
+  {
+    description: `Identify address field that has a placeholder, no associated label, and its autocomplete attribute is "off"`,
+    document: `<form>
+                <input id="targetElement" placeholder="Address" autocomplete="off">
+               </form>`,
+    elementId: "targetElement",
+    expectedReturnValue: ["street-address", {}],
+  },
+  {
+    description: `Identify address field that has a placeholder, no associated label, and the form's autocomplete attribute is "off"`,
+    document: `<form autocomplete="off">
+                <input id="targetElement" placeholder="Country">
+               </form>`,
+    elementId: "targetElement",
+    expectedReturnValue: ["country", {}],
   },
 ];
 
+add_setup(async function () {
+  Services.prefs.setStringPref(
+    "extensions.formautofill.creditCards.heuristics.fathom.testConfidence",
+    "1"
+  );
+
+  registerCleanupFunction(() => {
+    Services.prefs.clearUserPref(
+      "extensions.formautofill.creditCards.heuristics.fathom.testConfidence"
+    );
+  });
+});
+
 TESTCASES.forEach(testcase => {
-  add_task(async function() {
+  add_task(async function () {
     info("Starting testcase: " + testcase.description);
 
     let doc = MockDocument.createTestDocument(
@@ -287,10 +223,7 @@ TESTCASES.forEach(testcase => {
     );
 
     let element = doc.getElementById(testcase.elementId);
-    let value = FormAutofillHeuristics.getInfo(
-      element,
-      new FieldScanner([element], {})
-    );
+    let value = FormAutofillHeuristics.inferFieldInfo(element);
 
     Assert.deepEqual(value, testcase.expectedReturnValue);
     LabelUtils.clearLabelMap();
@@ -327,13 +260,8 @@ add_task(async function test_regexp_list() {
       document: `<select id="${label}"></select>`,
       elementId: label,
       expectedReturnValue: SUPPORT_LIST[label]
-        ? {
-            fieldName: SUPPORT_LIST[label],
-            section: "",
-            addressType: "",
-            contactType: "",
-          }
-        : null,
+        ? [SUPPORT_LIST[label], {}]
+        : [[], {}],
     };
     info(testcase.description);
     info(testcase.document);
@@ -343,10 +271,7 @@ add_task(async function test_regexp_list() {
     );
 
     let element = doc.getElementById(testcase.elementId);
-    let value = FormAutofillHeuristics.getInfo(
-      element,
-      new FieldScanner([element], {})
-    );
+    let value = FormAutofillHeuristics.inferFieldInfo(element);
 
     Assert.deepEqual(value, testcase.expectedReturnValue, label);
   }
@@ -358,7 +283,7 @@ add_task(async function test_autofill_creditCards_autocomplete_off_pref() {
                     <label for="targetElement"> Card Number</label>
                     <input id="targetElement" type="text">
                   </form>`;
-  let expected = null;
+  let expected = [[], {}];
   info(`Set pref so that credit card autofill respects autocomplete="off"`);
   Services.prefs.setBoolPref(
     FormAutofill.AUTOFILL_CREDITCARDS_AUTOCOMPLETE_OFF_PREF,
@@ -369,22 +294,14 @@ add_task(async function test_autofill_creditCards_autocomplete_off_pref() {
     document
   );
   let element = doc.getElementById("targetElement");
-  let value = FormAutofillHeuristics.getInfo(
-    element,
-    new FieldScanner([element], {})
-  );
+  let value = FormAutofillHeuristics.inferFieldInfo(element);
 
   Assert.deepEqual(value, expected);
   document = `<form>
                 <label for="targetElement"> Card Number</label>
                 <input id="targetElement" type="text">
               </form>`;
-  expected = {
-    fieldName: "cc-number",
-    section: "",
-    addressType: "",
-    contactType: "",
-  };
+  expected = ["cc-number", { fathomConfidence: 1 }];
   info(
     `Set pref so that credit card autofill does not respect autocomplete="off"`
   );
@@ -397,10 +314,7 @@ add_task(async function test_autofill_creditCards_autocomplete_off_pref() {
     document
   );
   element = doc.getElementById("targetElement");
-  value = FormAutofillHeuristics.getInfo(
-    element,
-    new FieldScanner([element], {})
-  );
+  value = FormAutofillHeuristics.inferFieldInfo(element);
 
   Assert.deepEqual(value, expected);
   Services.prefs.clearUserPref(
@@ -412,7 +326,7 @@ add_task(async function test_autofill_addresses_autocomplete_off_pref() {
   let document = `<form autocomplete="off">
                     <input id="given-name">
                   </form>`;
-  let expected = null;
+  let expected = [[], {}];
   info(`Set pref so that address autofill respects autocomplete="off"`);
   Services.prefs.setBoolPref(
     FormAutofill.AUTOFILL_ADDRESSES_AUTOCOMPLETE_OFF_PREF,
@@ -423,21 +337,13 @@ add_task(async function test_autofill_addresses_autocomplete_off_pref() {
     document
   );
   let element = doc.getElementById("given-name");
-  let value = FormAutofillHeuristics.getInfo(
-    element,
-    new FieldScanner([element], {})
-  );
+  let value = FormAutofillHeuristics.inferFieldInfo(element);
 
   Assert.deepEqual(value, expected);
   document = `<form>
                 <input id="given-name">
               </form>`;
-  expected = {
-    fieldName: "given-name",
-    section: "",
-    addressType: "",
-    contactType: "",
-  };
+  expected = ["given-name", {}];
   info(`Set pref so that address autofill does not respect autocomplete="off"`);
   Services.prefs.setBoolPref(
     FormAutofill.AUTOFILL_ADDRESSES_AUTOCOMPLETE_OFF_PREF,
@@ -448,10 +354,7 @@ add_task(async function test_autofill_addresses_autocomplete_off_pref() {
     document
   );
   element = doc.getElementById("given-name");
-  value = FormAutofillHeuristics.getInfo(
-    element,
-    new FieldScanner([element], {})
-  );
+  value = FormAutofillHeuristics.inferFieldInfo(element);
 
   Assert.deepEqual(value, expected);
   Services.prefs.clearUserPref(
