@@ -113,7 +113,6 @@
 #include "nsIComponentRegistrar.h"
 #include "nsISupportsPrimitives.h"
 #include "nsISimpleEnumerator.h"
-#include "nsMemory.h"
 #include "nsIXPConnect.h"
 #include "nsIXPCScriptable.h"
 #include "nsIObserver.h"
@@ -139,7 +138,6 @@
 #include "nsBaseHashtable.h"
 #include "nsHashKeys.h"
 #include "nsWrapperCache.h"
-#include "nsStringBuffer.h"
 #include "nsDeque.h"
 
 #include "nsIScriptSecurityManager.h"
@@ -150,7 +148,7 @@
 #include "xpcObjectHelper.h"
 
 #include "SandboxPrivate.h"
-#include "BackstagePass.h"
+#include "SystemGlobal.h"
 
 #ifdef XP_WIN
 // Nasty MS defines
@@ -177,22 +175,15 @@ extern const char XPC_SCRIPT_ERROR_CONTRACTID[];
 extern const char XPC_XPCONNECT_CONTRACTID[];
 
 /***************************************************************************/
-// Useful macros...
+// Helper function.
 
-#define XPC_STRING_GETTER_BODY(dest, src)   \
-  NS_ENSURE_ARG_POINTER(dest);              \
-  *dest = src ? moz_xstrdup(src) : nullptr; \
-  return NS_OK
+namespace xpc {
 
-// If IS_WN_CLASS for the JSClass of an object is true, the object is a
-// wrappednative wrapper, holding the XPCWrappedNative in its private slot.
-static inline bool IS_WN_CLASS(const JSClass* clazz) {
-  return clazz->isWrappedNative();
+inline bool IsWrappedNativeReflector(JSObject* obj) {
+  return JS::GetClass(obj)->isWrappedNative();
 }
 
-static inline bool IS_WN_REFLECTOR(JSObject* obj) {
-  return IS_WN_CLASS(JS::GetClass(obj));
-}
+}  // namespace xpc
 
 /***************************************************************************
 ****************************************************************************
@@ -211,7 +202,6 @@ class nsXPConnect final : public nsIXPConnect {
  public:
   // all the interface method declarations...
   NS_DECL_ISUPPORTS
-  NS_DECL_NSIXPCONNECT
 
   // non-interface implementation
  public:
@@ -261,26 +251,6 @@ class nsXPConnect final : public nsIXPConnect {
 
 /***************************************************************************/
 
-class XPCRootSetElem {
- public:
-  XPCRootSetElem() : mNext(nullptr), mSelfp(nullptr) {}
-
-  ~XPCRootSetElem() {
-    MOZ_ASSERT(!mNext, "Must be unlinked");
-    MOZ_ASSERT(!mSelfp, "Must be unlinked");
-  }
-
-  inline XPCRootSetElem* GetNextRoot() { return mNext; }
-  void AddToRootSet(XPCRootSetElem** listHead);
-  void RemoveFromRootSet();
-
- private:
-  XPCRootSetElem* mNext;
-  XPCRootSetElem** mSelfp;
-};
-
-/***************************************************************************/
-
 // In the current xpconnect system there can only be one XPCJSContext.
 // So, xpconnect can only be used on one JSContext within the process.
 
@@ -337,6 +307,10 @@ class XPCJSContext final : public mozilla::CycleCollectedJSContext,
   virtual void BeforeProcessTask(bool aMightBlock) override;
   virtual void AfterProcessTask(uint32_t aNewRecursionDepth) override;
 
+  // Relay to the CCGCScheduler instead of queuing up an idle runnable
+  // (as is done for workers in CycleCollectedJSContext).
+  virtual void MaybePokeGC() override;
+
   ~XPCJSContext();
 
   size_t SizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf);
@@ -375,6 +349,7 @@ class XPCJSContext final : public mozilla::CycleCollectedJSContext,
     IDX_CI,
     IDX_CR,
     IDX_CU,
+    IDX_SERVICES,
     IDX_WRAPPED_JSOBJECT,
     IDX_PROTOTYPE,
     IDX_EVAL,
@@ -400,6 +375,15 @@ class XPCJSContext final : public mozilla::CycleCollectedJSContext,
     IDX_INTERFACE_ID,
     IDX_INITIALIZER,
     IDX_PRINT,
+    IDX_FETCH,
+    IDX_CRYPTO,
+    IDX_INDEXEDDB,
+    IDX_STRUCTUREDCLONE,
+    IDX_LOCKS,
+#ifdef ENABLE_EXPLICIT_RESOURCE_MANAGEMENT
+    IDX_SUPPRESSED,
+    IDX_ERROR,
+#endif
     IDX_TOTAL_COUNT  // just a count of the above
   };
 
@@ -522,7 +506,7 @@ class XPCJSRuntime final : public mozilla::CycleCollectedJSRuntime {
   }
 
   virtual bool UsefulToMergeZones() const override;
-  void TraceNativeBlackRoots(JSTracer* trc) override;
+  void TraceAdditionalNativeBlackRoots(JSTracer* trc) override;
   void TraceAdditionalNativeGrayRoots(JSTracer* aTracer) override;
   void TraverseAdditionalNativeRoots(
       nsCycleCollectionNoteRootCallback& cb) override;
@@ -546,8 +530,7 @@ class XPCJSRuntime final : public mozilla::CycleCollectedJSRuntime {
   static void WeakPointerCompartmentCallback(JSTracer* trc,
                                              JS::Compartment* comp, void* data);
 
-  inline void AddVariantRoot(XPCTraceableVariant* variant);
-  inline void AddWrappedJSRoot(nsXPCWrappedJS* wrappedJS);
+  inline void AddSubjectToFinalizationWJS(nsXPCWrappedJS* wrappedJS);
 
   void DebugDump(int16_t depth);
 
@@ -581,12 +564,9 @@ class XPCJSRuntime final : public mozilla::CycleCollectedJSRuntime {
   JSObject* UnprivilegedJunkScope(const mozilla::fallible_t&);
 
   bool IsUnprivilegedJunkScope(JSObject*);
-  JSObject* LoaderGlobal();
 
   void DeleteSingletonScopes();
 
-  void SystemIsBeingShutDown();
-
  private:
   explicit XPCJSRuntime(JSContext* aCx);
 
@@ -628,13 +608,11 @@ class XPCJSRuntime final : public mozilla::CycleCollectedJSRuntime {
   bool mGCIsRunning;
   nsTArray<nsISupports*> mNativesToReleaseArray;
   bool mDoingFinalization;
-  XPCRootSetElem* mVariantRoots;
-  XPCRootSetElem* mWrappedJSRoots;
+  mozilla::LinkedList<nsXPCWrappedJS> mSubjectToFinalizationWJS;
   nsTArray<xpcGCCallback> extraGCCallbacks;
   JS::GCSliceCallback mPrevGCSliceCallback;
   JS::DoCycleCollectionCallback mPrevDoCycleCollectionCallback;
   mozilla::WeakPtr<SandboxPrivate> mUnprivilegedJunkScope;
-  JS::PersistentRootedObject mLoaderGlobal;
   RefPtr<AsyncFreeSnowWhite> mAsyncSnowWhiteFreer;
 
   friend class XPCJSContext;
@@ -817,6 +795,8 @@ class XPCWrappedNativeScope final
 
   bool AttachComponentsObject(JSContext* aCx);
 
+  bool AttachJSServices(JSContext* aCx);
+
   // Returns the JS object reflection of the Components object.
   bool GetComponentsJSObject(JSContext* cx, JS::MutableHandleObject obj);
 
@@ -1349,13 +1329,15 @@ class XPCWrappedNativeTearOff final {
 class XPCWrappedNative final : public nsIXPConnectWrappedNative {
  public:
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_NSIXPCONNECTJSOBJECTHOLDER
-  NS_DECL_NSIXPCONNECTWRAPPEDNATIVE
 
   NS_DECL_CYCLE_COLLECTION_CLASS(XPCWrappedNative)
 
+  JSObject* GetJSObject() override;
+
   bool IsValid() const { return mFlatJSObject.hasFlag(FLAT_JS_OBJECT_VALID); }
 
+  nsresult DebugDump(int16_t depth);
+
 #define XPC_SCOPE_WORD(s) (intptr_t(s))
 #define XPC_SCOPE_MASK (intptr_t(0x3))
 #define XPC_SCOPE_TAG (intptr_t(0x1))
@@ -1417,7 +1399,7 @@ class XPCWrappedNative final : public nsIXPConnectWrappedNative {
   void SetSet(already_AddRefed<XPCNativeSet> set) { mSet = set; }
 
   static XPCWrappedNative* Get(JSObject* obj) {
-    MOZ_ASSERT(IS_WN_REFLECTOR(obj));
+    MOZ_ASSERT(xpc::IsWrappedNativeReflector(obj));
     return JS::GetObjectISupports<XPCWrappedNative>(obj);
   }
 
@@ -1448,7 +1430,6 @@ class XPCWrappedNative final : public nsIXPConnectWrappedNative {
 
   static nsresult WrapNewGlobal(JSContext* cx, xpcObjectHelper& nativeHelper,
                                 nsIPrincipal* principal,
-                                bool initStandardClasses,
                                 JS::RealmOptions& aOptions,
                                 XPCWrappedNative** wrappedGlobal);
 
@@ -1583,16 +1564,15 @@ class XPCWrappedNative final : public nsIXPConnectWrappedNative {
 class nsXPCWrappedJS final : protected nsAutoXPTCStub,
                              public nsIXPConnectWrappedJSUnmarkGray,
                              public nsSupportsWeakReference,
-                             public XPCRootSetElem {
+                             public mozilla::LinkedListElement<nsXPCWrappedJS> {
  public:
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_NSIXPCONNECTJSOBJECTHOLDER
-  NS_DECL_NSIXPCONNECTWRAPPEDJS
-  NS_DECL_NSIXPCONNECTWRAPPEDJSUNMARKGRAY
   NS_DECL_NSISUPPORTSWEAKREFERENCE
 
-  NS_DECL_CYCLE_COLLECTION_SKIPPABLE_CLASS_AMBIGUOUS(nsXPCWrappedJS,
-                                                     nsIXPConnectWrappedJS)
+  NS_DECL_CYCLE_COLLECTION_SKIPPABLE_SCRIPT_HOLDER_CLASS_AMBIGUOUS(
+      nsXPCWrappedJS, nsIXPConnectWrappedJS)
+
+  JSObject* GetJSObject() override;
 
   // This method is defined in XPCWrappedJSClass.cpp to preserve VCS blame.
   NS_IMETHOD CallMethod(uint16_t methodIndex, const nsXPTMethodInfo* info,
@@ -1609,6 +1589,8 @@ class nsXPCWrappedJS final : protected nsAutoXPTCStub,
 
   nsISomeInterface* GetXPTCStub() { return mXPTCStub; }
 
+  nsresult DebugDump(int16_t depth);
+
   /**
    * This getter does not change the color of the JSObject meaning that the
    * object returned is not guaranteed to be kept alive past the next CC.
@@ -1668,8 +1650,6 @@ class nsXPCWrappedJS final : protected nsAutoXPTCStub,
     mRoot->mOuter = aNative;
   }
 
-  void TraceJS(JSTracer* trc);
-
   // This method is defined in XPCWrappedJSClass.cpp to preserve VCS blame.
   static void DebugDumpInterfaceInfo(const nsXPTInterfaceInfo* aInfo,
                                      int16_t depth);
@@ -1689,6 +1669,8 @@ class nsXPCWrappedJS final : protected nsAutoXPTCStub,
   void Unlink();
 
  private:
+  friend class nsIXPConnectWrappedJS;
+
   JS::Compartment* Compartment() const {
     return JS::GetCompartment(mJSObj.unbarrieredGet());
   }
@@ -2093,29 +2075,21 @@ using AutoMarkingWrappedNativeProtoPtr =
     TypedAutoMarkingPtr<XPCWrappedNativeProto>;
 
 /***************************************************************************/
-// in xpcvariant.cpp...
+// Definitions in XPCVariant.cpp.
 
 // {1809FD50-91E8-11d5-90F9-0010A4E73D9A}
-#define XPCVARIANT_IID                              \
-  {                                                 \
-    0x1809fd50, 0x91e8, 0x11d5, {                   \
-      0x90, 0xf9, 0x0, 0x10, 0xa4, 0xe7, 0x3d, 0x9a \
-    }                                               \
-  }
+#define XPCVARIANT_IID \
+  {0x1809fd50, 0x91e8, 0x11d5, {0x90, 0xf9, 0x0, 0x10, 0xa4, 0xe7, 0x3d, 0x9a}}
 
 // {DC524540-487E-4501-9AC7-AAA784B17C1C}
-#define XPCVARIANT_CID                               \
-  {                                                  \
-    0xdc524540, 0x487e, 0x4501, {                    \
-      0x9a, 0xc7, 0xaa, 0xa7, 0x84, 0xb1, 0x7c, 0x1c \
-    }                                                \
-  }
+#define XPCVARIANT_CID \
+  {0xdc524540, 0x487e, 0x4501, {0x9a, 0xc7, 0xaa, 0xa7, 0x84, 0xb1, 0x7c, 0x1c}}
 
 class XPCVariant : public nsIVariant {
  public:
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_NSIVARIANT
-  NS_DECL_CYCLE_COLLECTION_CLASS(XPCVariant)
+  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(XPCVariant)
 
   // If this class ever implements nsIWritableVariant, take special care with
   // the case when mJSVal is JSVAL_STRING, since we don't own the data in
@@ -2123,7 +2097,7 @@ class XPCVariant : public nsIVariant {
 
   // We #define and iid so that out module local code can use QI to detect
   // if a given nsIVariant is in fact an XPCVariant.
-  NS_DECLARE_STATIC_IID_ACCESSOR(XPCVARIANT_IID)
+  NS_INLINE_DECL_STATIC_IID(XPCVARIANT_IID)
 
   static already_AddRefed<XPCVariant> newVariant(JSContext* cx,
                                                  const JS::Value& aJSVal);
@@ -2135,6 +2109,7 @@ class XPCVariant : public nsIVariant {
    */
   JS::Value GetJSVal() const { return mJSVal; }
 
+ protected:
   /**
    * This getter does not change the color of the Value (if it represents a
    * JSObject) meaning that the value returned is not guaranteed to be kept
@@ -2148,6 +2123,7 @@ class XPCVariant : public nsIVariant {
 
   XPCVariant(JSContext* cx, const JS::Value& aJSVal);
 
+ public:
   /**
    * Convert a variant into a JS::Value.
    *
@@ -2160,35 +2136,18 @@ class XPCVariant : public nsIVariant {
   static bool VariantDataToJS(JSContext* cx, nsIVariant* variant,
                               nsresult* pErr, JS::MutableHandleValue pJSVal);
 
-  bool IsPurple() { return mRefCnt.IsPurple(); }
-
-  void RemovePurple() { mRefCnt.RemovePurple(); }
-
  protected:
-  virtual ~XPCVariant() = default;
+  virtual ~XPCVariant();
 
   bool InitializeData(JSContext* cx);
 
- protected:
+  void Cleanup();
+
   nsDiscriminatedUnion mData;
   JS::Heap<JS::Value> mJSVal;
   bool mReturnRawObject;
 };
 
-NS_DEFINE_STATIC_IID_ACCESSOR(XPCVariant, XPCVARIANT_IID)
-
-class XPCTraceableVariant : public XPCVariant, public XPCRootSetElem {
- public:
-  XPCTraceableVariant(JSContext* cx, const JS::Value& aJSVal)
-      : XPCVariant(cx, aJSVal) {
-    nsXPConnect::GetRuntimeInstance()->AddVariantRoot(this);
-  }
-
-  virtual ~XPCTraceableVariant();
-
-  void TraceJS(JSTracer* trc);
-};
-
 /***************************************************************************/
 // Utilities
 
@@ -2226,6 +2185,7 @@ struct GlobalProperties {
   bool ChromeUtils : 1;
   bool CSS : 1;
   bool CSSRule : 1;
+  bool CustomStateSet : 1;
   bool Directory : 1;
   bool Document : 1;
   bool DOMException : 1;
@@ -2238,8 +2198,11 @@ struct GlobalProperties {
   bool FormData : 1;
   bool Headers : 1;
   bool IOUtils : 1;
+  bool InspectorCSSParser : 1;
   bool InspectorUtils : 1;
   bool MessageChannel : 1;
+  bool MIDIInputMap : 1;
+  bool MIDIOutputMap : 1;
   bool Node : 1;
   bool NodeFilter : 1;
   bool PathUtils : 1;
@@ -2249,6 +2212,9 @@ struct GlobalProperties {
   bool Selection : 1;
   bool TextDecoder : 1;
   bool TextEncoder : 1;
+  bool TrustedHTML : 1;
+  bool TrustedScript : 1;
+  bool TrustedScriptURL : 1;
   bool URL : 1;
   bool URLSearchParams : 1;
   bool XMLHttpRequest : 1;
@@ -2265,6 +2231,7 @@ struct GlobalProperties {
   bool fetch : 1;
   bool storage : 1;
   bool structuredClone : 1;
+  bool locks : 1;
   bool indexedDB : 1;
   bool isSecureContext : 1;
   bool rtcIdentityProvider : 1;
@@ -2295,6 +2262,7 @@ class MOZ_STACK_CLASS OptionsBase {
   bool ParseJSString(const char* name, JS::MutableHandleString prop);
   bool ParseString(const char* name, nsCString& prop);
   bool ParseString(const char* name, nsString& prop);
+  bool ParseOptionalString(const char* name, mozilla::Maybe<nsString>& prop);
   bool ParseId(const char* name, JS::MutableHandleId id);
   bool ParseUInt32(const char* name, uint32_t* prop);
 
@@ -2322,7 +2290,8 @@ class MOZ_STACK_CLASS SandboxOptions : public OptionsBase {
         discardSource(false),
         metadata(cx),
         userContextId(0),
-        originAttributes(cx) {}
+        originAttributes(cx),
+        alwaysUseFdlibm(false) {}
 
   virtual bool Parse() override;
 
@@ -2332,6 +2301,7 @@ class MOZ_STACK_CLASS SandboxOptions : public OptionsBase {
   bool wantExportHelpers;
   bool isWebExtensionContentScript;
   JS::RootedObject proto;
+  mozilla::Maybe<nsString> sandboxContentSecurityPolicy;
   nsCString sandboxName;
   JS::RootedObject sameZoneAs;
   bool forceSecureContext;
@@ -2344,6 +2314,7 @@ class MOZ_STACK_CLASS SandboxOptions : public OptionsBase {
   JS::RootedValue metadata;
   uint32_t userContextId;
   JS::RootedObject originAttributes;
+  bool alwaysUseFdlibm;
 
  protected:
   bool ParseGlobalProperties();
@@ -2438,17 +2409,6 @@ JSObject* CreateGlobalObject(JSContext* cx, const JSClass* clasp,
                              nsIPrincipal* principal,
                              JS::RealmOptions& aOptions);
 
-// Modify the provided compartment options, consistent with |aPrincipal| and
-// with globally-cached values of various preferences.
-//
-// Call this function *before* |aOptions| is used to create the corresponding
-// global object, as not all of the options it sets can be modified on an
-// existing global object.  (The type system should make this obvious, because
-// you can't get a *mutable* JS::RealmOptions& from an existing global
-// object.)
-void InitGlobalObjectOptions(JS::RealmOptions& aOptions,
-                             nsIPrincipal* aPrincipal);
-
 // Finish initializing an already-created, not-yet-exposed-to-script global
 // object.  This will attach a Components object (if necessary) and call
 // |JS_FireOnNewGlobalObject| (if necessary).
@@ -2485,8 +2445,17 @@ nsresult EvalInSandbox(JSContext* cx, JS::HandleObject sandbox,
 nsresult GetSandboxMetadata(JSContext* cx, JS::HandleObject sandboxArg,
                             JS::MutableHandleValue rval);
 
-nsresult SetSandboxMetadata(JSContext* cx, JS::HandleObject sandboxArg,
-                            JS::HandleValue metadata);
+[[nodiscard]] nsresult SetSandboxMetadata(JSContext* cx,
+                                          JS::HandleObject sandboxArg,
+                                          JS::HandleValue metadata);
+
+[[nodiscard]] nsresult SetSandboxLocaleOverride(JSContext* cx,
+                                                JS::HandleObject sandboxArg,
+                                                const char* locale);
+
+[[nodiscard]] nsresult SetSandboxTimezoneOverride(JSContext* cx,
+                                                  JS::HandleObject sandboxArg,
+                                                  const char* timezone);
 
 bool CreateObjectIn(JSContext* cx, JS::HandleValue vobj,
                     CreateObjectInOptions& options,
@@ -2846,6 +2815,11 @@ void InitializeValue(const nsXPTType& aType, void* aValue);
 void DestructValue(const nsXPTType& aType, void* aValue,
                    uint32_t aArrayLen = 0);
 
+bool SandboxCreateCrypto(JSContext* cx, JS::Handle<JSObject*> obj);
+bool SandboxCreateFetch(JSContext* cx, JS::Handle<JSObject*> obj);
+bool SandboxCreateStructuredClone(JSContext* cx, JS::Handle<JSObject*> obj);
+bool SandboxCreateLocks(JSContext* cx, JS::Handle<JSObject*> obj);
+
 }  // namespace xpc
 
 namespace mozilla {
